from dubidoc.enum import HttpMethod


__all__ = ('DocumentAPI',)


class DocumentAPI:
    PATH = 'documents'

    def __init__(self, client):
        self.client = client

    def list(self):
        return self.client.make_request(HttpMethod.GET, self.PATH)

    def create(self, body, organization_id=None):
        """
        filename — назва файлу з розширенням (опціонально використовується для додаткової валідації)
        file — base64
        title — відображувана назва в кабінеті користувача, емейл листі тощо (рекомендується передавати назву файлу без розширення, хоча це не критично)
        """
        return self.client.make_request(HttpMethod.POST, self.PATH, body)

    def get(self, document_id):
        path = f'{self.PATH}/{document_id}'
        return self.client.make_request(HttpMethod.GET, path)

    def edit(self, document_id, body):
        path = f'{self.PATH}/{document_id}'
        return self.client.make_request(HttpMethod.PUT, path, body)

    def delete(self, document_id):
        """
        Title: Видалення документа
        Description: Увага, видалити документ можливо лише за умови, що він не був підписаний або надісланий отримувачу
        """
        path = f'{self.PATH}/{document_id}'
        return self.client.make_request(HttpMethod.DELETE, path)

    def download(self):
        raise NotImplementedError

    def unarchive(self):
        raise NotImplementedError

    def participants(self):
        raise NotImplementedError

    def sign(self):
        raise NotImplementedError

    def signatures(self):
        raise NotImplementedError

    def archive(self):
        raise NotImplementedError



from dubidoc.enum import HttpMethod

__all__ = ('DocumentLinkAPI',)


class DocumentLinkAPI:
    PATH = 'documents/{}/links'

    def __init__(self, client):
        self.client = client

    def generate_public_link(self, document_id, body):
        path = self.PATH.format(document_id)
        return self.client.make_request(HttpMethod.POST, path, body)

    def revoke_public_link(self, document_id):
        path = self.PATH.format(document_id)
        return self.client.make_request(HttpMethod.DELETE, path)



import logging
from urllib.parse import urljoin

import requests

from dubidoc import __version__
from dubidoc._modules import (
    AccessTokenAPI,
    AuthenticationAPI,
    DeviceAPI,
    DocumentAPI,
    DocumentLinkAPI,
    ParticipantAPI,
    DownloadAPI,
)
from dubidoc.enum import HttpMethod


logger = logging.getLogger('dubidoc')


class DubidocAPIClient:
    """
    - https://my.dubidoc.com.ua/auth - production URL
    - https://docs-stage.navkolo.one/auth - staging URL
    """

    API_VERSION = 'v1'
    DEFAULT_HEADERS = {
        'accept': 'application/json',
        'content-type': 'application/json',
        'user-agent': f'python-dubidoc/{__version__} | (https://github.com/DmytroLitvinov/python-dubidoc)',
    }

    def __init__(self, api_token: str, environment: str = 'stage'):
        self.api_token = api_token
        if environment == 'stage':
            self.base_url = 'https://docs-stage.navkolo.one'
        else:
            self.base_url = 'https://my.dubidoc.com.ua'
        # FIXME: The prefix should be "api/v1" instead of "api/api/v1"
        #  Dubidoc will notify when it will be fixed
        self.prefix = f'api/api/{self.API_VERSION}/'
        self.endpoint = urljoin(self.base_url, self.prefix)

        # Modules
        self.document_api = DocumentAPI(self)
        self.document_link_api = DocumentLinkAPI(self)
        self.participant_api = ParticipantAPI(self)
        self.download_api = DownloadAPI(self)

        if environment == 'stage':
            self.access_token_api = AccessTokenAPI(self)
            self.authentication_api = AuthenticationAPI(self)
            self.device_api = DeviceAPI(self)
            # Not yet implemented anot probably not needed
            # self.organization_user_api = OrganizationUserAPI(self)
            # self.organization_api = OrganizationAPI(self)
            # self.shortcode_api = ShortcodeAPI(self)

    def _get_headers(self):
        headers = self.DEFAULT_HEADERS.copy()
        headers.update({'X-Access-Token': f'{self.api_token}'})
        return headers

    def make_request(self, method: HttpMethod, path: str, body: dict = {}):
        """
        Fetches the given path in the Dubidoc API.
        :param method: HTTP method
        :param path: Api path
        :param body: body of request
        :return: Serialized server response or error
        """
        url = urljoin(self.endpoint, path)
        headers = self._get_headers()

        logger.debug(f'Making {method.value} request to {url} with headers {headers} and body {body}')
        # https://github.com/psf/requests/issues/3070
        response = requests.request(method.value, url, headers=headers, json=body, timeout=10)
        logger.debug(f'Received response with status code {response.status_code} and body {response.text}')

        return response.json()



from dubidoc.enum import HttpMethod


__all__ = ('AuthenticationAPI',)


class AuthenticationAPI:
    def __init__(self, client):
        self.client = client

    def get_code(self, login):
        """
        Title: Отримати код авторизації
        Description: Метод відправки тимчасового коду для авторизації через email
        """
        path = self.PATH
        body = {'login': login}
        return self.client.make_request(HttpMethod.POST, path, body)

    def get_token(self, login, code):
        """
        Title: Отримати авторизаційний токен
        Description: Метод для отримання токену користувача на основі email та тимчасового коду авторизації
        """
        path = 'auth/get-token'
        body = {'login': login, 'code': code}
        return self.client.make_request(HttpMethod.POST, path, body)

    def refresh(self, refresh_token):
        """
        Title: Оновити авторизаційний токен
        Description: Продовжити дію авторизаційного токену користувача на основі refresh токену
        :param token_id:
        :return:
        """
        path = 'token/refresh'
        body = {'refresh_token': refresh_token}
        return self.client.delete(HttpMethod.POST, path, body)



from dubidoc.enum import HttpMethod


__all__ = ('DownloadAPI',)


class DownloadAPI:
    PATH = 'downloads'

    def __init__(self, client):
        self.client = client

    def bulk_uploads(self, body):
        path = self.PATH
        return self.client.make_request(HttpMethod.POST, path, body)

    def download(self, download_id):
        path = f'{self.PATH}/{download_id}/download'
        return self.client.make_request(HttpMethod.GET, path)

    def check_status(self, download_id):
        path = f'{self.PATH}/{download_id}/status'
        return self.client.make_request(HttpMethod.GET, path)



from dubidoc.enum import HttpMethod

__all__ = ('ParticipantAPI',)


class ParticipantAPI:
    def __init__(self, client):
        self.client = client

    def add_participant(self, document_id, body):
        path = 'documents/{}/participants'.format(document_id)
        return self.client.make_request(HttpMethod.POST, path, body)

    def remove_participant(self, participant_id):
        """
        It sounds like it should be DELETE 'documents/{}/participants/{}', but
        each participant has own unique id, so it should be 'participants/{}'.
        You can retrieve participant_id from the response of 'add_participant' method.
        """
        path = 'participants/{}'.format(participant_id)
        return self.client.make_request(
            HttpMethod.DELETE,
            path,
        )



from dubidoc.enum import HttpMethod

__all__ = ('DeviceAPI',)


class DeviceAPI:
    def __init__(self, client):
        self.client = client

    def register(self, token):
        """
        Title: Зареєструвати пристрій для відправки сповіщень
        Description: Зареєструвати device token який отриманий через Firebase Cloud Messaging для відправки нотифікацій користувачу
        """
        path = 'devices'
        body = {'token': token}
        return self.client.make_request(HttpMethod.POST, path, body)

    def unregister(self, token):
        """
        Title: Відписати пристрій від отримання сповіщень
        Description: Скасувати відправку сповіщень для device token
        """
        path = f'devices/{token}'
        return self.client.make_request(HttpMethod.DELETE, path)



from dubidoc.enum import HttpMethod

__all__ = ('AccessTokenAPI',)


class AccessTokenAPI:
    PATH = 'access-tokens'

    def __init__(self, client):
        self.client = client

    def get_tokens(self):
        """
        Отримати список всіх згенерованих Вами API-ключів (токенів)
        """
        path = self.PATH
        return self.client.make_request(HttpMethod.GET, path)

    def generate_token(self, title):
        """
        Генерування статичного API-ключа (токену) для доступу до API
        :param title:
        """
        path = self.PATH
        return self.client.make_request(HttpMethod.POST, path, body={'title': title})

    def revoke_token(self, token_id):
        """
        Зробити обраний API-ключ (токен) недійсним
        :param token_id:
        """
        path = f'{self.PATH}/{token_id}'
        return self.client.make_request(HttpMethod.DELETE, path)
